@@ -5,10 +5,13 @@
 # CLAM: Computational Linguistics Application Mediator
 # -- CLAM Webservice --
 #       by Maarten van Gompel (proycon)
-#       http://ilk.uvt.nl/clam
-#       http://ilk.uvt.nl/~mvgompel
+#       http://proycon.github.com/clam
+#
+#       Centre for Language Studies
+#       Radboud University Nijmegen
+#
 #       Induction for Linguistic Knowledge Research Group
-#       Universiteit van Tilburg
+#       Tilburg University
 #       
 #       Licensed under GPLv3
 #
@@ -43,17 +46,27 @@ from clam.common.util import globsymlinks, setdebug, setlog, printlog, printdebu
 import clam.config.defaults as settings #will be overridden by real settings later
 settings.STANDALONEURLPREFIX = ''
 
+
+
+class CustomForbidden(web.webapi.HTTPError):
+     """Custom `403 Forbidden` error, because later versions of web.py seem to have disallowed custom messages. Are we violating the standard?"""
+     
+     def __init__(self, message="forbidden"):
+         status = "403 Forbidden"
+         headers = {'Content-Type': 'text/html'}         
+         web.webapi.HTTPError.__init__(self, status, headers, message)    
+         
 try:
     import MySQLdb
 except ImportError:
-    print >>sys.stderr, "WARNING: No MySQL support available in your version of Python! Install python-mysql "
+    print >>sys.stderr, "WARNING: No MySQL support available in your version of Python! Install python-mysql if you plan on using MySQL for authentication"
 
 #Maybe for later: HTTPS support
 #web.wsgiserver.CherryPyWSGIServer.ssl_certificate = "path/to/ssl_certificate"
 #web.wsgiserver.CherryPyWSGIServer.ssl_private_key = "path/to/ssl_private_key"
 
 
-VERSION = '0.7.9'
+VERSION = '0.8.0'
 
 DEBUG = False
     
@@ -98,13 +111,13 @@ def userdb_lookup_dict(user, realm):
 
 def userdb_lookup_mysql(user, realm):
     printdebug("Looking up user " + user + " in MySQL")
-    host,port, mysqluser,passwd, database, table = validate_users_mysql()
-    db = MySQLdb.connect(host=host,user=mysqluser,passwd=passwd,db=database, charset='utf-8', use_unicode=True)
+    host,port, mysqluser,passwd, database, table, userfield, passwordfield = validate_users_mysql()
+    db = MySQLdb.connect(host=host,user=mysqluser,passwd=passwd,db=database, charset='utf8', use_unicode=True)
     cursor = db.cursor()
     #simple protection against mysql injection
     user = user.replace("'","") 
     user = user.replace(";","")
-    sql = "SELECT `user`, `password` FROM `" + table + "` WHERE user='" + user + "' LIMIT 1"
+    sql = "SELECT `" + userfield + "`, `" + passwordfield + "` FROM `" + table + "` WHERE " + userfield + "='" + user + "' LIMIT 1"
     cursor.execute(sql)
     password = None
     while True:
@@ -141,16 +154,24 @@ def validate_users_mysql():
     if 'password' in settings.USERS_MYSQL:
         password = settings.USERS_MYSQL['password']
     else:
-        raise Exception("No MySQL user defined in USERS_MYSQL")            
+        raise Exception("No MySQL password defined in USERS_MYSQL")            
     if 'database' in settings.USERS_MYSQL:
         database = settings.USERS_MYSQL['database']
     else:
-        raise Exception("No MySQL user defined in USERS_MYSQL")     
+        raise Exception("No MySQL database defined in USERS_MYSQL")     
     if 'table' in settings.USERS_MYSQL:
         table = settings.USERS_MYSQL['table']
     else:
-        raise Exception("No MySQL user defined in USERS_MYSQL")   
-    return host,port, user,password, database, table
+        raise Exception("No MySQL table defined in USERS_MYSQL")   
+    if 'userfield' in settings.USERS_MYSQL:
+        userfield = settings.USERS_MYSQL['userfield']
+    else:
+        userfield = "username"
+    if 'passwordfield' in settings.USERS_MYSQL:
+        passwordfield = settings.USERS_MYSQL['passwordfield']
+    else:
+        passwordfield = "password"
+    return host,port, user,password, database, table, userfield, passwordfield
 
 #requirelogin = lambda x: x
 #if settings.USERS:
@@ -241,7 +262,7 @@ class TestInterface(object):
     
     @RequireLogin()
     def GET(self, user = None):
-        raise web.webapi.Forbidden('Test error response')
+        raise CustomForbidden('Test error response')
             
 
 class CLAMService(object):
@@ -268,8 +289,10 @@ class CLAMService(object):
         printlog("Starting CLAM WebService, version " + str(VERSION) + " ...")
         if not settings.ROOT or not os.path.isdir(settings.ROOT):
             error("Specified root path " + settings.ROOT + " not found")
-        elif not settings.COMMAND.split(" ")[0] or os.system("which " + settings.COMMAND.split(" ")[0] + "> /dev/null 2> /dev/null") != 0:
-            error("Specified command " + settings.COMMAND.split(" ")[0] + " not found or without execute permission")
+        elif not settings.COMMAND.split(" ")[0] or not os.path.exists( settings.COMMAND.split(" ")[0]): 
+            error("Specified command " + settings.COMMAND.split(" ")[0] + " not found")
+        elif not os.access(settings.COMMAND.split(" ")[0], os.X_OK):
+            error("Specified command " + settings.COMMAND.split(" ")[0] + " is not executable")            
         elif not settings.PROFILES:
             error("No profiles were defined in settings module!")
         elif not settings.PARAMETERS:
@@ -412,10 +435,10 @@ class Project(object):
         """Create project skeleton if it does not already exist (static method)"""
         if not user: user = 'anonymous'
         if not Project.validate(project):
-            raise web.webapi.Forbidden('Invalid project ID')
+            raise CustomForbidden('Invalid project ID')
         printdebug("Checking if " + settings.ROOT + "projects/" + user + '/' + project + " exists") 
         if not project:
-            raise web.webapi.Forbidden('No project name') 
+            raise CustomForbidden('No project name') 
         if not os.path.isdir(settings.ROOT + "projects/" + user):
             printlog("Creating user directory '" + user + "'")
             os.mkdir(settings.ROOT + "projects/" + user)
@@ -688,10 +711,10 @@ class Project(object):
         if errors:
             #There are parameter errors, return 403 response with errors marked
             printlog("There are parameter errors, not starting.")
-            raise web.webapi.Forbidden(unicode(self.response(user, project, parameters)))
+            raise CustomForbidden(unicode(self.response(user, project, parameters)))
         elif not matchedprofiles:
             printlog("No profiles matching, not starting.")
-            raise web.webapi.Forbidden(unicode(self.response(user, project, parameters, "No profiles matching input and parameters, unable to start. Are you sure you added all necessary input files and set all necessary parameters?")))
+            raise CustomForbidden(unicode(self.response(user, project, parameters, "No profiles matching input and parameters, unable to start. Are you sure you added all necessary input files and set all necessary parameters?")))
         else:
             #write clam.xml output file
             render = web.template.render(settings.CLAMDIR + '/templates')
@@ -916,7 +939,7 @@ class OutputFileHandler(object):
                 if os.path.isfile(Project.path(project, user) + "output/" + project + ".zip"):
                     os.unlink(Project.path(project, user) + "output/" + project + ".zip")
             else:
-                raise web.webapi.Forbidden('Invalid archive format') #TODO: message won't show
+                raise CustomForbidden('Invalid archive format') #TODO: message won't show
 
             path = Project.path(project, user) + "output/" + project + "." + format
             
@@ -958,7 +981,7 @@ class InputFileHandler(object):
         
         if filename.strip('/') == "":
             #this is a request for the index
-            raise web.webapi.Forbidden()
+            raise CustomForbidden()
         if len(raw) >= 2:
             #This MAY be a viewer/metadata request, check:
             if os.path.isfile(Project.path(project, user) + 'input/' +  "/".join(raw[:-1])):
@@ -1051,7 +1074,7 @@ class InputFileHandler(object):
                                     inputtemplate = t
                                     break                    
                 if not inputsource:
-                    raise web.webapi.Forbidden("No such inputsource exists")
+                    raise CustomForbidden("No such inputsource exists")
                 if not inputtemplate:
                     for profile in settings.PROFILES:
                         for t in profile.input:
@@ -1078,7 +1101,7 @@ class InputFileHandler(object):
                 else:
                     assert False                    
             else:
-                raise web.webapi.Forbidden("No filename or inputsource specified")
+                raise CustomForbidden("No filename or inputsource specified")
         else:
             #Simply forward to addfile
             return self.addfile(project,filename,user, postdata)
@@ -1127,7 +1150,7 @@ class InputFileHandler(object):
             
         for seq, inputfile in Project.inputindexbytemplate(project, user, inputtemplate):
             if inputtemplate.unique:
-                raise web.webapi.Forbidden("You have already submitted a file of this type, you can only submit one. Delete it first. (Inputtemplate=" + inputtemplate.id + ", unique=True)") #(it will have to be explicitly deleted by the client first)
+                raise CustomForbidden("You have already submitted a file of this type, you can only submit one. Delete it first. (Inputtemplate=" + inputtemplate.id + ", unique=True)") #(it will have to be explicitly deleted by the client first)
             else:
                 if seq >= nextseq:
                     nextseq = seq + 1 #next available sequence number
@@ -1147,20 +1170,20 @@ class InputFileHandler(object):
         else:    
             if inputtemplate.filename:
                 if filename != inputtemplate.filename:
-                    raise web.webapi.Forbidden("Specified filename must the filename dictated by the inputtemplate, which is " + inputtemplate.filename)
+                    raise CustomForbidden("Specified filename must the filename dictated by the inputtemplate, which is " + inputtemplate.filename)
                 #TODO LATER: add support for calling this with an actual number instead of #
             if inputtemplate.extension:
                 if filename[-len(inputtemplate.extension) - 1:].lower() == '.' + inputtemplate.extension.lower():
                     #good, extension matches (case independent). Let's just make sure the case is as defined exactly by the inputtemplate
                     filename = filename[:-len(inputtemplate.extension) - 1] +  '.' + inputtemplate.extension
                 else:
-                    raise web.webapi.Forbidden("Specified filename does not have the extension dictated by the inputtemplate ("+inputtemplate.extension+")") #403
+                    raise CustomForbidden("Specified filename does not have the extension dictated by the inputtemplate ("+inputtemplate.extension+")") #403
             
         if inputtemplate.onlyinputsource and (not 'inputsource' in postdata or not postdata['inputsource']):
-            raise web.webapi.Forbidden("Adding files for this inputtemplate must proceed through inputsource") #403
+            raise CustomForbidden("Adding files for this inputtemplate must proceed through inputsource") #403
             
         if 'converter' in postdata and postdata['converter'] and not postdata['converter'] in [ x.id for x in inputtemplate.converters]:            
-                raise web.webapi.Forbidden("Invalid converter specified: " + postdata['converter']) #403
+                raise CustomForbidden("Invalid converter specified: " + postdata['converter']) #403
 
         #Very simple security, prevent breaking out the input dir
         filename = filename.replace("..","")
@@ -1197,10 +1220,10 @@ class InputFileHandler(object):
                     if s.id.lower() == postdata['inputsource'].lower():
                         inputsource = s
                 if not inputsource:
-                    raise web.webapi.Forbidden("Specified inputsource '" + postdata['inputsource'] + "' does not exist for inputtemplate '"+inputtemplate.id+"'")
+                    raise CustomForbidden("Specified inputsource '" + postdata['inputsource'] + "' does not exist for inputtemplate '"+inputtemplate.id+"'")
             sourcefile = os.path.basename(inputsource.path)
         else:
-            raise web.webapi.Forbidden("No file, url or contents specified!")
+            raise CustomForbidden("No file, url or contents specified!")
             
 
 
@@ -1399,7 +1422,7 @@ class InputFileHandler(object):
                             f.write(postdata['contents'])
                             f.close()
                         except UnicodeError:
-                            raise web.webapi.Forbidden("Input file " + str(filename) + " is not in the expected encoding!")
+                            raise CustomForbidden("Input file " + str(filename) + " is not in the expected encoding!")
                     elif 'inputsource' in postdata and postdata['inputsource']:                
                         #Copy (symlink!) from preinstalled data
                         os.symlink(inputsource.path, Project.path(project, user) + 'input/' + filename)
@@ -1494,11 +1517,11 @@ class InputFileHandler(object):
         if fatalerror:
             #fatal error return error message with 403 code
             printlog('Fatal Error during upload: ' + fatalerror)
-            raise web.webapi.Forbidden(head + fatalerror)
+            raise CustomForbidden(head + fatalerror)
         elif errors:
             #parameter errors, return XML output with 403 code
             printdebug('There were paramameter errors during upload!')
-            raise web.webapi.Forbidden(output)
+            raise CustomForbidden(output)
         else:
             #everything ok, return XML output with 200 code
             return output
@@ -1877,8 +1900,8 @@ def set_defaults(HOST = None, PORT = None):
     
     if 'ROOT' in settingkeys and settings.ROOT and not settings.ROOT[-1] == "/":
         settings.ROOT += "/" #append slash
-    if not 'USER' in settingkeys:
-        settings.USER = None
+    if not 'USERS' in settingkeys:
+        settings.USERS = None
     if not 'ADMINS' in settingkeys:
         settings.ADMINS = []
     if not 'PROJECTS_PUBLIC' in settingkeys:
@@ -1909,6 +1932,8 @@ def set_defaults(HOST = None, PORT = None):
         settings.DISPATCHER = 'clamdispatcher.py'
     if not 'REALM' in settingkeys:
         settings.REALM = settings.SYSTEM_ID
+    if not 'DIGESTOPAQUE' in settingkeys:
+        settings.DIGESTOPAQUE = "%032x" % random.getrandbits(128)
     if not 'ENABLEWEBAPP' in settingkeys:
         settings.ENABLEWEBAPP = True
     elif settings.ENABLEWEBAPP is False:
@@ -1974,6 +1999,22 @@ def test_dirs():
     if not settings.USERS and not settings.USERS_MYSQL and not settings.PREAUTHHEADER:
             warning("No user authentication enabled, this is not recommended for production environments!")
 
+def test_version():
+    global VERSION
+    #Check version
+    req = str(settings.REQUIRE_VERSION).split('.')
+    ver = str(VERSION).split('.')
+
+    uptodate = True
+    for i in range(0,len(req)):
+        if i < len(ver):
+            if req[i] > ver[i]:
+                uptodate = False
+                break
+            elif ver[i] > req[i]:
+                break
+    if not uptodate:
+        error("Version mismatch: at least " + str(settings.REQUIRE_VERSION) + " is required")
 
 
 if __name__ == "__main__":
@@ -2035,23 +2076,20 @@ if __name__ == "__main__":
 
     import_string = "import " + settingsmodule + " as settings"
     exec import_string
+    
+    try:
+        if settings.DEBUG:
+            DEBUG = True
+            setdebug(True)
+    except:    
+        pass
+    try: 
+        if settings.LOGFILE:
+            setlogfile(settings.LOGFILE)
+    except:
+        pass
 
-    #Check version
-    req = str(settings.REQUIRE_VERSION).split('.')
-    ver = str(VERSION).split('.')
-
-    uptodate = True
-    for i in range(0,len(req)):
-        if i < len(ver):
-            if req[i] > ver[i]:
-                uptodate = False
-                break
-            elif ver[i] > req[i]:
-                break
-    if not uptodate:
-        error("Version mismatch: at least " + str(settings.REQUIRE_VERSION) + " is required")
-
-
+    test_version()
     set_defaults(HOST,PORT)
     if HOST:
         settings.HOST = HOST
@@ -2080,10 +2118,10 @@ if __name__ == "__main__":
     #if USERS:
     #    requirelogin = digestauth.auth(lambda x: USERS[x], realm=SYSTEM_ID)
     if settings.USERS:
-        auth = clam.common.digestauth.auth(userdb_lookup_dict, settings.REALM, printdebug, settings.STANDALONEURLPREFIX)
+        auth = clam.common.digestauth.auth(userdb_lookup_dict, settings.REALM, printdebug, settings.STANDALONEURLPREFIX, True, "","Unauthorized",8, settings.DIGESTOPAQUE)
     elif settings.USERS_MYSQL:
         validate_users_mysql()
-        auth = clam.common.digestauth.auth(userdb_lookup_mysql, settings.REALM, printdebug, settings.STANDALONEURLPREFIX)    
+        auth = clam.common.digestauth.auth(userdb_lookup_mysql, settings.REALM, printdebug, settings.STANDALONEURLPREFIX,True,"","Unauthorized",8, settings.DIGESTOPAQUE)    
         
 
     
@@ -2092,7 +2130,7 @@ if __name__ == "__main__":
 
 def run_wsgi(settings_module):
     """Run CLAM in WSGI mode"""
-    global settingsmodule, auth
+    global settingsmodule, auth, DEBUG
     #import_string = "import " + settingsmodule + " as settings"
     #exec import_string
     printdebug("Initialising WSGI service")
@@ -2102,16 +2140,30 @@ def run_wsgi(settings_module):
     globals()['settings'] = settings_module
     settingsmodule = settings_module.__name__
     
+        
+    try:
+        if settings.DEBUG:
+            DEBUG = True
+            setdebug(True)
+    except:    
+        pass
+    try: 
+        if settings.LOGFILE:
+            setlogfile(settings.LOGFILE)
+    except:
+        pass
+    
+    test_version()
     setlog(None)
     set_defaults(None,None)
     test_dirs()
 
     if settings.USERS:
-        auth = clam.common.digestauth.auth(userdb_lookup_dict, settings.REALM, printdebug, settings.URLPREFIX)
+        auth = clam.common.digestauth.auth(userdb_lookup_dict, settings.REALM, printdebug, settings.URLPREFIX, True, "","Unauthorized",8, settings.DIGESTOPAQUE)
         printdebug("Initialised authentication")    
     elif settings.USERS_MYSQL:
         validate_users_mysql()
-        auth = clam.common.digestauth.auth(userdb_lookup_mysql, settings.REALM, printdebug, settings.URLPREFIX)            
+        auth = clam.common.digestauth.auth(userdb_lookup_mysql, settings.REALM, printdebug, settings.URLPREFIX, True, "","Unauthorized",8, settings.DIGESTOPAQUE)            
         printdebug("Initialised MySQL authentication")
         
     service = CLAMService('wsgi')
