@@ -22,7 +22,7 @@ class MalformedAuthenticationHeader(Exception): pass
 
 class auth(object):
     """A decorator class implementing digest authentication (RFC 2617)"""
-    def __init__(self,  getHA1,  realm="Protected",  printdebug = None, urlprefix = None, tolerateIE = True, redirectURL = '/newuser',  unauthHTML = None,  nonceSkip = 0,  lockTime = 20,  nonceLife = 350,  tries=3,  domain=[]):
+    def __init__(self,  getHA1,  realm="Protected",  printdebug = None, urlprefix = None, tolerateIE = True, redirectURL = '/unauth',  unauthHTML = None,  nonceSkip = 0, staticopaque=None, lockTime = 20,  nonceLife = 350,  tries=3,  domain=[]):
         """Creates a decorator specific to a particular web application. 
             getHA1: a function taking the arguments (username, realm), and returning digestauth.H(username:realm:password), or
                             throwing KeyError if no such user
@@ -42,8 +42,13 @@ class auth(object):
         self.urlprefix = urlprefix
         self.outstandingNonces = NonceMemory()
         self.user_status = {}
-        self.opaque = "%032x" % random.getrandbits(128)
-        if self.printdebug: self.printdebug("DEBUG generated opaque: " + str(self.opaque))
+        if staticopaque:
+            if self.printdebug: self.printdebug("DEBUG set opaque from settings: '" + str(staticopaque) + "'")
+            self.opaque = staticopaque
+        else:
+            if self.printdebug: self.printdebug("DEBUG generated opaque: '" + str(self.opaque)+ "'")  
+            self.opaque = "%032x" % random.getrandbits(128)
+        
         self.webpy2 = web.__version__.startswith('0.2')
     g401HTML = """
 <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
@@ -89,7 +94,8 @@ class auth(object):
             if nonceReaction == 2:
                 # Client sent a nonce we've never heard of before
                 if self.printdebug: self.printdebug("AUTH DEBUG unknownnonce")
-                return self.denyBadRequest()
+                #return self.send401UnauthorizedResponse() #test
+                return self.denyBadRequest() #original
             if nonceReaction == 3:
                 # Client sent an old nonce.  Give the client a new one, and ask to authenticate again before continuing.
                 if self.printdebug: self.printdebug("AUTH DEBUG oldnonce")
