@@ -22,8 +22,8 @@ from clam.common.data import *
 from clam.common.converters import *
 from clam.common.digestauth import pwhash
 from sys import path 
-from os import uname
-
+from os import uname, environ
+from base64 import b64decode as D
 
 REQUIRE_VERSION = 0.7
 
@@ -34,15 +34,36 @@ SYSTEM_ID = "frog"
 SYSTEM_NAME = "Frog"
 SYSTEM_DESCRIPTION = "Frog is a suite containing a tokeniser, Part-of-Speech tagger, lemmatiser, morphological analyser, shallow parser, and dependency parser for Dutch, developed at Tilburg University. It is the successor of Tadpole."
 
-# ================ Root directory for CLAM ===============
+
+USERS = None
+
+# ================ Server specific configuration for CLAM ===============
 host = uname()[1]
-if host == 'aurora' or host == 'roma': #proycon's laptop/server
+if host == 'galactica' or host == 'roma': #proycon's laptop/server
     CLAMDIR = "/home/proycon/work/clam"
     ROOT = "/home/proycon/work/frog.clam/"
     PORT = 9001
     BINDIR = "/usr/local/bin/"
+    USERS = { 'proycon': pwhash('proycon', SYSTEM_ID, 'secret') }
     #URLPREFIX = 'frog'
-else:
+elif host == 'applejack': #Nijmegen
+    CLAMDIR = "/scratch2/www/webservices-lst/live/repo/clam"
+    ROOT = "/scratch2/www/webservices-lst/live/writable/frog/"
+    HOST = "webservices-lst.science.ru.nl"
+    PORT = 80
+    URLPREFIX = "frog"
+    BINDIR = "/vol/customopt/uvt-ru/bin/"
+    USERS_MYSQL = {
+        'host': 'mysql-clamopener.science.ru.nl', 
+        'user': 'clamopener',        
+        'password': D(open(environ['CLAMOPENER_KEYFILE']).read().strip()),
+        'database': 'clamopener',
+        'table': 'clamusers_clamusers'
+    }
+    DEBUG = True
+    REALM = "WEBSERVICES-LST"
+    DIGESTOPAQUE = open(environ['CLAM_DIGESTOPAQUEFILE']).read().strip()
+elif host == 'echo' or host == 'nomia' or host == 'echo.uvt.nl' or host == 'nomia.uvt.nl': #Tilburg
     #Assuming ILK server
     CLAMDIR = "/var/www/clam"
     ROOT = "/var/www/clamdata/frog/"
@@ -51,13 +72,9 @@ else:
     URLPREFIX = 'frog'
     WEBSERVICEGHOST = 'ws'
     BINDIR = "/var/www/bin/"
-
-#=========== Definition of users and passwords =====================
-
-#Users and passwords
-
-USERS = None #Enable this instead if you want no authentication
-#USERS = { 'username': pwhash('username', SYSTEM_ID, 'secret') } #Using pwhash and plaintext password in code is not secure!! 
+else:
+    raise Exception("I don't know where I'm running from! Got " + host)
+ 
 
 
 
